import boto3
import time
import sys
import os

def s3_folder_exists_hourly(bucket, folder, event_date, event_hour):
    session = boto3.session.Session()
    client = session.client('s3')

    prefix = folder+'/event_date='+event_date+'/event_hour='+event_hour
    response = client.list_objects_v2(
        Bucket=bucket,
        Prefix=prefix,
    )
    return len(response['Contents']) > 0

def s3_folder_exists_daily(bucket, folder, event_date):
    session = boto3.session.Session()
    client = session.client('s3')

    prefix = folder+'/event_date='+event_date
    response = client.list_objects_v2(
        Bucket=bucket,
        Prefix=prefix,
    )
    return len(response['Contents']) > 0

def add_partition_hourly(database, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour):
    client = boto3.client("athena")
    sql = "ALTER TABLE " + table_name + " ADD PARTITION (event_date= '" \
          + event_date + "', event_hour= '" + event_hour + "') LOCATION " \
          + "'s3://" + s3_bucket + "/" + folder + "/event_date=" \
          + event_date + "/event_hour=" + event_hour + "/'"
    print(sql)


    response = client.start_query_execution(
        QueryString=sql,
        QueryExecutionContext={
            'Database': database
        },
        ResultConfiguration={
            'OutputLocation': 's3://' + athena_result_bucket
        },
        WorkGroup=os.environ['ATHENA_WORKGROUP']
    )

def add_partition_daily(database, table_name, s3_bucket, folder, athena_result_bucket, event_date):
    client = boto3.client("athena")
    sql = "ALTER TABLE " + table_name + " ADD PARTITION (event_date= '" \
          + event_date + "') LOCATION " \
          + "'s3://" + s3_bucket + "/" + folder + "/event_date=" \
          + event_date + "/'"
    print(sql)

    response = client.start_query_execution(
        QueryString=sql,
        QueryExecutionContext={
            'Database': database
        },
        ResultConfiguration={
            'OutputLocation': 's3://' + athena_result_bucket
        },
        WorkGroup=os.environ['ATHENA_WORKGROUP']
    )

def lambda_handler(event, context):
    env = os.environ['ENV']
    database_name = os.environ['DATABASE_NAME']
    athena_result_bucket = f"{os.environ['ATHENA_OUTPUT_BUCKET']}/results"
    s3_bucket = os.environ['FLINK_RESULT_BUCKET']

    now = time.gmtime()
    event_date = time.strftime("%Y-%m-%d", now)
    event_hour = time.strftime("%H", now)

    table_name = 'flink_cbd'
    folder = 'cbd/event-time'
    try:
        if s3_folder_exists_hourly(s3_bucket, folder, event_date, event_hour):
            add_partition_hourly(database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except:
        pass

    table_name = 'flink_cbd_response_alarm'
    folder = 'response-alarm/processing-time'
    try:
        if event_hour == '23' and s3_folder_exists_daily(s3_bucket, folder, event_date):
            add_partition_daily(database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date)
    except:
        pass

    table_name = 'flink_cbd_security_event'
    folder = 'security-event/event-time'
    try:
        if s3_folder_exists_hourly(s3_bucket, folder, event_date, event_hour):
            add_partition_hourly(database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except:
        pass
